package com.elab.spring.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.elab.core.utils.ObjectUtils;
import com.elab.log.utils.EventProcessUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.*;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

/**
 * http调用工具
 *
 * @author Liukx
 * @create 2018-04-26 14:45
 * @email liukx@elab-plus.com
 **/
public class RestTemplateUtils {


    private Logger logger = LoggerFactory.getLogger(RestTemplateUtils.class);
    public RestTemplate restTemplate;

    private HttpHeaders headers;

    private RestTemplate getRestTemplate() {
        return restTemplate;
    }

    public void setRestTemplate(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    public HttpHeaders getHeaders() {
        return headers;
    }

    public void setHeaders(HttpHeaders headers) {
        this.headers = headers;
    }

    public RestTemplateUtils() {
        //复杂构造函数的使用
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        // 设置超时
        requestFactory.setConnectTimeout(1000);
        // 读取超时
        requestFactory.setReadTimeout(5000);

        //利用复杂构造器可以实现超时设置，内部实际实现为 HttpClient
        restTemplate = new RestTemplate(requestFactory);

        //设置HTTP请求头信息，实现编码等
        headers = new HttpHeaders();
        MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
        headers.setContentType(type);
        headers.add("Accept", MediaType.APPLICATION_JSON.toString());
    }

    /**
     * post数据放送
     *
     * @param url
     * @param reqParam
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> T post(String url, Object reqParam, Class<T> clazz) {
        EventProcessUtils.writeThirdParty(url);
        logger.info(" URL : " + url);
        if (reqParam != null) {
            logger.info(" RequestData : " + ObjectUtils.objectParseJsonStr(reqParam));
        }

        //利用容器实现数据封装，发送
        HttpEntity<Object> entity = new HttpEntity<Object>(reqParam, headers);
        T o = restTemplate.postForObject(url, entity, clazz);
        if (o != null) {
            logger.info(" ResponseData : " + ObjectUtils.objectParseJsonStr(o));
        }
        return o;
    }

    /**
     * get请求发送
     *
     * @param url
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> T get(String url, Class<T> clazz) {
        EventProcessUtils.writeThirdParty(url);
        logger.info(" URL : " + url);
        //利用容器实现数据封装，发送
        T o = restTemplate.getForObject(url, clazz);
        return o;
    }

    /**
     * 文件上传接口
     *
     * @param url
     * @param reqParam
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> T postUpload(String url, Object reqParam, Class<T> clazz) {
        logger.info(" URL : " + url);
        logger.info(" RequestData : " + ObjectUtils.objectParseJsonStr(reqParam));
        //利用容器实现数据封装，发送
        HttpEntity<Object> entity = new HttpEntity<Object>(reqParam, new HttpHeaders());
        ResponseEntity<T> exchange = restTemplate.exchange(url, HttpMethod.POST, entity, clazz);
        if (exchange != null && exchange.getBody() != null) {
            logger.info(" ResponseData : " + ObjectUtils.objectParseJsonStr(exchange.getBody()));
        }
        return (T) exchange;
    }

    public static void main(String[] args) {
        String param = "{\n" +
                "\t\"pageno\":\"1\",\n" +
                "\t\"pagesize\":\"10\",\n" +
                "\t\"houseid\":\"102\",\n" +
                "\t\"time\":\"2017-04-13\"\n" +
                "}";
        JSONObject jsonObject = JSON.parseObject(param);
        RestTemplateUtils restTemplateUtils = new RestTemplateUtils();
//        JSONModel post = restTemplateUtils.post("http://localhost:9005/skyforest/dd/pageList", jsonObject, JSONModel.class);
//        System.out.println(JSON.toJSONString(post));
    }

}
